"""Copyright 2016-2017 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""


from traceutils.record.record import Record


class RegisterRecord(Record):
    """Register trace record."""

    REGISTER = "register"
    VALUE = "value"
    VALUE_SIZE = "value_size"

    REGISTER_FMT = "(?P<%s>\d+) (?P<%s>\w+) R (?P<%s>\S+) (?P<%s>[a-fA-F0-9]+)"\
                   % (Record.TIME, Record.SCALE, REGISTER, VALUE)

    def __init__(self, time, scale, register, value):
        """Default constructor."""
        Record.__init__(self, time, scale)
        self[self.REGISTER] = register
        self[self.VALUE] = int(value, 16)
        self[self.VALUE_SIZE] = len(value)

    def get_register(self):
        """Returns the register name."""
        return self[self.REGISTER]

    def get_value(self):
        """Returns the hexadecimal value written to the register (64 bits maximum)."""
        return self[self.VALUE]

    def get_value_size(self):
        """Returns the size in characters of the input hexadecimal value."""
        return self[self.VALUE_SIZE]

    @staticmethod
    def get_format():
        """Returns regular expression that encodes register record format."""
        return RegisterRecord.REGISTER_FMT

    def to_string(self):
        """Returns the string representation of this record, None fields are removed."""
        field_list = [self.get_time(), self.get_scale(), "R", self.get_register(),
                      ("%x" % self.get_value()).zfill(self.get_value_size())]
        return " ".join(str(item) for item in field_list)
