"""Copyright 2016-2017 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""


import logging


class Trace:
    """A trace that is a sequence of strings that represent events."""

    def __init__(self, trace_file_path):
        """Default constructor."""

        self._trace_file = open(trace_file_path, 'r')

        self._lines = self._trace_file.readlines()
        self._line_num = len(self._lines)
        assert self._line_num, ['File %s is empty' % trace_file_path]

        self._line_index = 0
        self._name = trace_file_path

    def name(self):
        """Returns trace name."""
        return self._name

    def get_trace_length(self):
        """Returns the number of lines for this trace."""
        return self._line_num

    def has_next_event(self):
        """Checks whether an object contains more events."""

        return self._line_index < self._line_num

    def get_next_event(self):
        """Returns the current event of this trace."""

        line = self._lines[self._line_index]
        self._line_index += 1
        return line

    def peek_next_event(self):
        """Takes the current event for the trace without removing it from the trace."""

        return self._lines[self._line_index]

    def delete_next_event(self):
        """Removes the current event from the trace."""

        self._line_index += 1

    def dump_rest_lines(self):
        """Prints the rest of trace events."""
        logging.debug('Dump the rest lines of trace: %s' % self.name())
        rest_lines = self._lines[self._line_index:]
        for rest_line in rest_lines:
            logging.info(rest_line.rstrip())
